@@ -12,7 +12,7 @@ use Carp;
 
 use vars qw($VERSION @ISA);
 
-$VERSION = '2.11';
+$VERSION = '2.12';
 
 use HTML::Parser;
 @ISA = qw(HTML::Parser);
@@ -118,14 +118,11 @@ sub start {
     my $ts = $self->_enter_table(@_);
     $ts->tree($res[0]) if @res;
   }
-
-  # Rows and cells are next.
-  if ($self->{_in_a_table}) {
+  elsif ($self->{_in_a_table}) {
+    # Rows and cells are next.
     my $ts = $self->current_table;
-    my $skiptag = 0;
     if ($_[0] eq 'tr') {
       $ts->_enter_row;
-      ++$skiptag;
     }
     elsif ($_[0] eq 'td' || $_[0] eq 'th') {
       $ts->_enter_cell(@_);
@@ -134,24 +131,28 @@ sub start {
       my $cspan = $attrs{colspan} || 1;
       $ts->_rasterizer->($ts->row_count, $rspan, $cspan);
       $ts->_anchor_item(@res);
-      ++$skiptag;
     }
-    if ($self->{keep_html} && !$skiptag) {
-      $self->text($_[3]);
+    elsif (! TREE() && $ts->{in_cell}) {
+      if ($self->{keep_html}) {
+        # capture full text of tag
+        $self->text($_[3]);
+      }
+      elsif ($_[0] eq 'br' && $self->{br_translate}) {
+        # Replace <br> with newlines if requested
+        $self->text("\n");
+      }
     }
   }
 
-  # Replace <br> with newlines if requested
-  if ($_[0] eq 'br' && $self->{br_translate} && !$self->{keep_html}) {
-    $self->text("\n");
-  }
-
   @res;
 } # end start
 
 sub end {
   my $self = shift;
-  my @res = $self->SUPER::end(@_) if TREE();
+  my @res;
+ 
+  @res = $self->SUPER::end(@_) if TREE();
+
   if ($self->{_in_a_table}) {
     my $ts = $self->current_table;
     if ($_[0] eq 'td' || $_[0] eq 'th') {
@@ -163,28 +164,38 @@ sub end {
     elsif ($_[0] eq 'table') {
       $self->_exit_table;
     }
-    unless (TREE()) {
-      $self->text($_[1]) if $self->{keep_html} && $ts->{in_cell};
+    elsif (! TREE()) {
+      if ($self->{keep_html} && $ts->{in_cell}) {
+        # capture full text of tag
+        $self->text($_[1]);
+      }
     }
   }
+
   @res;
-}
+} # end end
 
 sub text {
   my $self = shift;
-  my @res = $self->SUPER::text(@_) if TREE();
-  if ($self->{_in_a_table} && !TREE()) {
+  my @res;
+
+  if (TREE()) {
+    @res = $self->SUPER::text(@_);
+  }
+  elsif ($self->{_in_a_table}) {
     my $ts = $self->current_table;
-    return unless $ts->{in_cell};
-    if ($self->{decode} && !$self->{keep_html}) {
-      $ts->_add_text(decode_entities($_[0]));
-    }
-    else {
-      $ts->_add_text($_[0]);
+    if ($ts->{in_cell}) {
+      if ($self->{decode} && !$self->{keep_html}) {
+        $ts->_add_text(decode_entities($_[0]));
+      }
+      else {
+        $ts->_add_text($_[0]);
+      }
     }
   }
+
   @res;
-}
+} # end text
 
 sub parse {
   my $self = shift;
@@ -886,7 +897,6 @@ sub _emsg {
     my $r = shift;
     $r <= $#{$self->{grid}}
       or croak "row $r out of range ($#{$self->{grid}})\n";
-    my @ri = $self->row_indices;
     my @row = $self->_slice_and_normalize_row(
                 $self->{grid}[($self->row_indices)[$r]]
               );
@@ -977,7 +987,7 @@ sub _emsg {
     if ($self->{headers}) {
       # First we order the original column counts by taking a hash slice
       # based on the original header order. The resulting original
-      # column numbers are mapped to the actual content indicies since
+      # column numbers are mapped to the actual content indices since
       # we could have a sparse slice.
       my %order;
       foreach (keys %{$self->{hits}}) {
@@ -1154,7 +1164,7 @@ sub _emsg {
       $current_row->[$col] = [$rspan, $cspan];
       $col_spinner[$col] = $rspan - 1;
 
-      # if this is an embedded placment (not a trailing element), use up
+      # if this is an embedded placement (not a trailing element), use up
       # the cspan
       if ($col < $#$current_row) {
         my $offset = 1;
@@ -1372,7 +1382,7 @@ starting with 0. Providing both a I<depth> and a I<count> will
 uniquely specify a table within a document.
 
 I<Attributes> match based on the attributes of the html E<lt>tableE<gt>
-tag, for example, boder widths or background color.
+tag, for example, border widths or background color.
 
 Each of the I<Headers>, I<Depth>, I<Count>, and I<Attributes>
 specifications are cumulative in their effect on the overall extraction.
@@ -1402,10 +1412,10 @@ another. In this case, HTML::TableExtract is a subclass of HTML::Parser:
 
   use HTML::TableExtract;
 
-Alternativevly, tables can be extracted as HTML::ElementTable
+Alternatively, tables can be extracted as HTML::ElementTable
 structures, which are in turn embedded in an HTML::Element tree
 representing the entire HTML document. Embedded tables are not decoupled
-from one another since this tree structure must be manitained. In this
+from one another since this tree structure must be maintained. In this
 case, HTML::TableExtract is a subclass of HTML::TreeBuilder (itself a
 subclass of HTML:::Parser):
 
@@ -1513,7 +1523,7 @@ matching header above them. Disabling this also disables C<automap>.
 Disabled by default, and only applicable when header constraints have
 been specified, C<keep_headers> will retain the matching header row as
 the first row of table data when enabled. This option has no effect if
-extracting into an element tree tructure. In any case, the header row is
+extracting into an element tree structure. In any case, the header row is
 accessible from the table method C<hrow()>.
 
 =item gridmap
